/**
 * Copyright 2013, WebGate Consulting AG
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at:
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * permissions and limitations under the License.
 */
package org.openntf.xpt.core.json;

import java.io.IOException;

import com.ibm.commons.util.io.json.JsonException;
import com.ibm.domino.services.util.JsonWriter;

public enum JSONEmptyValueStrategy {
	NULL, EMPTYSTRING, NOPROPERTY;

	public static JSONEmptyValueStrategy getStrategy(boolean showEmptyValue, boolean showEmptyValueAsString) {
		if (showEmptyValueAsString) {
			return EMPTYSTRING;
		}
		if (showEmptyValue) {
			return NULL;
		}
		return NOPROPERTY;
	}

	public void writeJSONValue(JsonWriter jsWriter) throws IOException, JsonException {
		if (this == JSONEmptyValueStrategy.EMPTYSTRING) {
			jsWriter.outStringLiteral("");
			return;
		}
		if (this == JSONEmptyValueStrategy.NULL) {
			jsWriter.outNull();
		}
		
	}
}
